define(
	'OrderWizard.Module.PaymentMethod.Creditcard.Extension'
,	[
		'Utils'
	,	'OrderWizard.Module.PaymentMethod'
	,	'jQuery'
	,	'underscore'
	,	'Profile.Model'
	]
,	function (
		Utils
	,	OrderWizardModulePaymentMethod
	,	jQuery
	,	_
	,	ProfileModel
	)
{
	'use strict';
	_.extend(OrderWizardModulePaymentMethodCreditcard.prototype,
	{
		//@method submit
		submit: function ()
		{
			// This order is bing payed with some other method (Gift Cert probably)
			if (this.wizard.hidePayment())
			{
				return jQuery.Deferred().resolve();
			}

			var self = this;

			if (this.requireccsecuritycode)
			{
				this.isSecurityNumberInvalid = false;
				// we need to store this temporally (frontend) in case a module in the same step
				// fails validation, making the credit card section re-rendered.
				// We don't want the user to have to type the security number multiple times
				this.ccsecuritycode = this.$('input[name="ccsecuritycode"]').val();
			}

			// if we are adding a new credit card
			if (this.creditcardView)
			{
				var fake_event = jQuery.Event('click', {
						target: this.creditcardView.$('form').get(0)
					})
				,	result = this.creditcardView.saveForm(fake_event);

				if (!result || result.frontEndValidationError)
				{
					// There were errors so we return a rejected promise
					return jQuery.Deferred().reject({
						errorCode: 'ERR_CHK_INCOMPLETE_CREDITCARD'
					,	errorMessage: _('The Credit Card is incomplete').translate()
					});
				}

				var save_result = jQuery.Deferred();

				result.done(function (model)
				{
					self.creditcardView = null;

					ProfileModel.getInstance().get('creditcards').add(model, {
						silent: true
					});

					self.setCreditCard({
						model: model
					});

					save_result.resolve();
				}).fail(function (error)
				{
					save_result.reject(error.responseJSON);
				});

				return save_result;
			}
			// if there are already credit cards
			else
			{
				this.setSecurityNumber();

				OrderWizardModulePaymentMethod.prototype.submit.apply(this, arguments);

				this.setCreditCard({ model: this.paymentMethod.get('creditcard') });

				return this.isValid().fail(function (error)
				{
					if (error === self.securityNumberErrorMessage)
					{
						self.isSecurityNumberInvalid = true;
					}

				}).done(function ()
				{
					self.isSecurityNumberInvalid = false;

				}).always(function ()
				{
					// Call self.render() instead of self._render() because the last one didn't asign the events to the DOM
					self.render();
				});
			}
		}
	});
});
